part of masamune.localize;

/// Class to translate
/// Execute the [initialize()] method and load the csv file
/// After that, display the translated text by executing the [get()] method
class Localize {
  static const String _defaultLanguage = "en";
  static const String _systemPath = "localize";
  static DataCollection _collection;
  static DataDocument get _document {
    if( __document == null ) __document = _collection[language];
    return __document;
  }
  static DataDocument __document;
  
  /// Language settings for translation
  static String get language {
    if( isNotEmpty(_language) ) return _language;
    return _defaultLanguage;
  }
  /// Set language settings
  /// 
  /// [language]: Language
  static set language( String language ){
    _language = language;
    __document = _collection[_language];
  }
  static String _language;

  /// True if initialization has been completed
  static bool get isInitialized => _isInitialized;
  static bool _isInitialized = false;

  /// Get translated text
  /// 
  /// [key]: Key to get
  /// [defaultValue]: Initial value
  static String get( String key, {String defaultValue}){
    if( _document == null || !_document.containsKey(key) ) return defaultValue ?? key;
    return _document[key] ?? defaultValue ?? key;
  }

  /// Initialize localization
  /// 
  /// [path]: Translation file path
  static Future initialize( String path ) async {
    if( isEmpty( path ) ){
      Log.error("CSV File path is empty.");
      return Future.error("CSV File path is empty.");
    }
    String csv = await rootBundle.loadString(path);
    if( isEmpty( csv ) ){
      Log.error("CSV data is empty.");
      return Future.error("CSV data is empty.");
    }
    _collection = DataCollection( _systemPath );
    List<List> converted = const CsvToListConverter().convert(csv);
    for( int y = 1; y < converted.length; y++ ){
      List line = converted[y];
      if( line == null ) continue;
      for( int x = 1; x < line.length; x++ ){
        String tmp = line[x];
        if( isEmpty(tmp) ) continue;
        if( y == 1 ){
          List<String> langs = tmp.split(Const.colon);
          _collection.add(DataDocument(Paths.child(_systemPath, langs[0])));
        } else {
          String key = line[0];
          String tmp = line[x];
          if( isEmpty(tmp) ) continue;
          if( isEmpty(key) || key.startsWith("#") ) continue;
          DataDocument doc = _collection[x-1];
          doc[key] = tmp;
        }
      }
    }
    String language = await Devicelocale.currentLocale;
    if( isEmpty(language) ) return;
    _language = language.split(Const.underbar)[0];
    __document = _collection[language];
    _isInitialized = true;
  }
}